/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and others contributors as indicated 
 * by the @authors tag. All rights reserved. 
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors. 
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * (C) 2005-2006,
 * @author JBoss Inc.
 */
package org.jboss.soa.esb.qa.performance.ebws_proxy;

import org.jboss.soa.esb.actions.AbstractActionLifecycle;
import org.jboss.soa.esb.helpers.ConfigTree;
import org.jboss.soa.esb.message.Message;

public class Envelope extends AbstractActionLifecycle {
	protected ConfigTree _config;

	public Envelope(ConfigTree config) {
		_config = config;
	}

	public Message noOperation(Message message) {
		return message;
	}

	public Message add(Message message) throws Exception {
		String payload = (String) message.getBody().get();
		message
				.getBody()
				.add(
						"<soapenv:Envelope xmlns:soapenv='http://schemas.xmlsoap.org/soap/envelope/' xmlns:m='http://plain_ws.helloworld'><soapenv:Header/><soapenv:Body>"
								+ payload.replaceFirst("ebws_proxy/helloworld", "plain_ws.helloworld")
								+ "</soapenv:Body></soapenv:Envelope>");
		//System.out.println("Request: " + (String) message.getBody().get());
		return message;
	}

	public Message remove(Message message) throws Exception {
		String m = new String((byte[])message.getBody().get());
		//System.out.println("Original Response: " + m);
		int bodyBeg = m.indexOf("<env:Body>");
		int bodyEnd = m.indexOf("</env:Body>");
		String payload = m.substring(bodyBeg + 10, bodyEnd);
		//System.out.println("Response: " + payload);
		message.getBody().add(payload);
		return message;
	}

	public void exceptionHandler(Message message, Throwable exception) {
		logHeader();
		exception.printStackTrace();
		System.out.println("!ERROR!");
		System.out.println(exception.getMessage());
		System.out.println("For Message: ");
		System.out.println((byte[]) message.getBody().get());
		logFooter();
	}

	// This makes it easier to read on the console
	private void logHeader() {
		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&\n");
	}

	private void logFooter() {
		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&\n");
	}

}
